import datetime
from http import HTTPStatus
from typing import Any
from uuid import UUID

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...models.paginated_response_run_response import PaginatedResponseRunResponse
from ...models.run_field import RunField
from ...models.run_status import RunStatus
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    workflow_id: None | Unset | UUID = UNSET,
    machine_id: None | Unset | UUID = UNSET,
    session_id: None | Unset | UUID = UNSET,
    status: None | RunStatus | Unset = UNSET,
    created_at_from: datetime.datetime | None | Unset = UNSET,
    created_at_to: datetime.datetime | None | Unset = UNSET,
    fields: list[RunField] | None | Unset = UNSET,
    skip: int | Unset = 0,
    limit: int | Unset = 100,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    json_workflow_id: None | str | Unset
    if isinstance(workflow_id, Unset):
        json_workflow_id = UNSET
    elif isinstance(workflow_id, UUID):
        json_workflow_id = str(workflow_id)
    else:
        json_workflow_id = workflow_id
    params["workflow_id"] = json_workflow_id

    json_machine_id: None | str | Unset
    if isinstance(machine_id, Unset):
        json_machine_id = UNSET
    elif isinstance(machine_id, UUID):
        json_machine_id = str(machine_id)
    else:
        json_machine_id = machine_id
    params["machine_id"] = json_machine_id

    json_session_id: None | str | Unset
    if isinstance(session_id, Unset):
        json_session_id = UNSET
    elif isinstance(session_id, UUID):
        json_session_id = str(session_id)
    else:
        json_session_id = session_id
    params["session_id"] = json_session_id

    json_status: None | str | Unset
    if isinstance(status, Unset):
        json_status = UNSET
    elif isinstance(status, RunStatus):
        json_status = status.value
    else:
        json_status = status
    params["status"] = json_status

    json_created_at_from: None | str | Unset
    if isinstance(created_at_from, Unset):
        json_created_at_from = UNSET
    elif isinstance(created_at_from, datetime.datetime):
        json_created_at_from = created_at_from.isoformat()
    else:
        json_created_at_from = created_at_from
    params["created_at_from"] = json_created_at_from

    json_created_at_to: None | str | Unset
    if isinstance(created_at_to, Unset):
        json_created_at_to = UNSET
    elif isinstance(created_at_to, datetime.datetime):
        json_created_at_to = created_at_to.isoformat()
    else:
        json_created_at_to = created_at_to
    params["created_at_to"] = json_created_at_to

    json_fields: list[str] | None | Unset
    if isinstance(fields, Unset):
        json_fields = UNSET
    elif isinstance(fields, list):
        json_fields = []
        for fields_type_0_item_data in fields:
            fields_type_0_item = fields_type_0_item_data.value
            json_fields.append(fields_type_0_item)

    else:
        json_fields = fields
    params["fields"] = json_fields

    params["skip"] = skip

    params["limit"] = limit

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/runs",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> HTTPValidationError | PaginatedResponseRunResponse | None:
    if response.status_code == 200:
        response_200 = PaginatedResponseRunResponse.from_dict(response.json())

        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[HTTPValidationError | PaginatedResponseRunResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    workflow_id: None | Unset | UUID = UNSET,
    machine_id: None | Unset | UUID = UNSET,
    session_id: None | Unset | UUID = UNSET,
    status: None | RunStatus | Unset = UNSET,
    created_at_from: datetime.datetime | None | Unset = UNSET,
    created_at_to: datetime.datetime | None | Unset = UNSET,
    fields: list[RunField] | None | Unset = UNSET,
    skip: int | Unset = 0,
    limit: int | Unset = 100,
) -> Response[HTTPValidationError | PaginatedResponseRunResponse]:
    """List Runs

     List all runs for the authenticated organization.

    Supports pagination and filtering by workflow, machine, and status.
    Returns runs with their associated workflow and machine data.

    Args:
        workflow_id (None | Unset | UUID): Filter by workflow ID
        machine_id (None | Unset | UUID): Filter by machine ID
        session_id (None | Unset | UUID): Filter by session ID
        status (None | RunStatus | Unset): Filter by run status
        created_at_from (datetime.datetime | None | Unset): Filter runs created at or after this
            ISO timestamp (UTC)
        created_at_to (datetime.datetime | None | Unset): Filter runs created at or before this
            ISO timestamp (UTC)
        fields (list[RunField] | None | Unset): Optional list of fields to include per run. Always
            includes: id, workflow_id, machine_id, status, created_at. Provide multiple 'fields='
            params to include more.
        skip (int | Unset):  Default: 0.
        limit (int | Unset):  Default: 100.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | PaginatedResponseRunResponse]
    """

    kwargs = _get_kwargs(
        workflow_id=workflow_id,
        machine_id=machine_id,
        session_id=session_id,
        status=status,
        created_at_from=created_at_from,
        created_at_to=created_at_to,
        fields=fields,
        skip=skip,
        limit=limit,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    workflow_id: None | Unset | UUID = UNSET,
    machine_id: None | Unset | UUID = UNSET,
    session_id: None | Unset | UUID = UNSET,
    status: None | RunStatus | Unset = UNSET,
    created_at_from: datetime.datetime | None | Unset = UNSET,
    created_at_to: datetime.datetime | None | Unset = UNSET,
    fields: list[RunField] | None | Unset = UNSET,
    skip: int | Unset = 0,
    limit: int | Unset = 100,
) -> HTTPValidationError | PaginatedResponseRunResponse | None:
    """List Runs

     List all runs for the authenticated organization.

    Supports pagination and filtering by workflow, machine, and status.
    Returns runs with their associated workflow and machine data.

    Args:
        workflow_id (None | Unset | UUID): Filter by workflow ID
        machine_id (None | Unset | UUID): Filter by machine ID
        session_id (None | Unset | UUID): Filter by session ID
        status (None | RunStatus | Unset): Filter by run status
        created_at_from (datetime.datetime | None | Unset): Filter runs created at or after this
            ISO timestamp (UTC)
        created_at_to (datetime.datetime | None | Unset): Filter runs created at or before this
            ISO timestamp (UTC)
        fields (list[RunField] | None | Unset): Optional list of fields to include per run. Always
            includes: id, workflow_id, machine_id, status, created_at. Provide multiple 'fields='
            params to include more.
        skip (int | Unset):  Default: 0.
        limit (int | Unset):  Default: 100.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | PaginatedResponseRunResponse
    """

    return sync_detailed(
        client=client,
        workflow_id=workflow_id,
        machine_id=machine_id,
        session_id=session_id,
        status=status,
        created_at_from=created_at_from,
        created_at_to=created_at_to,
        fields=fields,
        skip=skip,
        limit=limit,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    workflow_id: None | Unset | UUID = UNSET,
    machine_id: None | Unset | UUID = UNSET,
    session_id: None | Unset | UUID = UNSET,
    status: None | RunStatus | Unset = UNSET,
    created_at_from: datetime.datetime | None | Unset = UNSET,
    created_at_to: datetime.datetime | None | Unset = UNSET,
    fields: list[RunField] | None | Unset = UNSET,
    skip: int | Unset = 0,
    limit: int | Unset = 100,
) -> Response[HTTPValidationError | PaginatedResponseRunResponse]:
    """List Runs

     List all runs for the authenticated organization.

    Supports pagination and filtering by workflow, machine, and status.
    Returns runs with their associated workflow and machine data.

    Args:
        workflow_id (None | Unset | UUID): Filter by workflow ID
        machine_id (None | Unset | UUID): Filter by machine ID
        session_id (None | Unset | UUID): Filter by session ID
        status (None | RunStatus | Unset): Filter by run status
        created_at_from (datetime.datetime | None | Unset): Filter runs created at or after this
            ISO timestamp (UTC)
        created_at_to (datetime.datetime | None | Unset): Filter runs created at or before this
            ISO timestamp (UTC)
        fields (list[RunField] | None | Unset): Optional list of fields to include per run. Always
            includes: id, workflow_id, machine_id, status, created_at. Provide multiple 'fields='
            params to include more.
        skip (int | Unset):  Default: 0.
        limit (int | Unset):  Default: 100.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | PaginatedResponseRunResponse]
    """

    kwargs = _get_kwargs(
        workflow_id=workflow_id,
        machine_id=machine_id,
        session_id=session_id,
        status=status,
        created_at_from=created_at_from,
        created_at_to=created_at_to,
        fields=fields,
        skip=skip,
        limit=limit,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    workflow_id: None | Unset | UUID = UNSET,
    machine_id: None | Unset | UUID = UNSET,
    session_id: None | Unset | UUID = UNSET,
    status: None | RunStatus | Unset = UNSET,
    created_at_from: datetime.datetime | None | Unset = UNSET,
    created_at_to: datetime.datetime | None | Unset = UNSET,
    fields: list[RunField] | None | Unset = UNSET,
    skip: int | Unset = 0,
    limit: int | Unset = 100,
) -> HTTPValidationError | PaginatedResponseRunResponse | None:
    """List Runs

     List all runs for the authenticated organization.

    Supports pagination and filtering by workflow, machine, and status.
    Returns runs with their associated workflow and machine data.

    Args:
        workflow_id (None | Unset | UUID): Filter by workflow ID
        machine_id (None | Unset | UUID): Filter by machine ID
        session_id (None | Unset | UUID): Filter by session ID
        status (None | RunStatus | Unset): Filter by run status
        created_at_from (datetime.datetime | None | Unset): Filter runs created at or after this
            ISO timestamp (UTC)
        created_at_to (datetime.datetime | None | Unset): Filter runs created at or before this
            ISO timestamp (UTC)
        fields (list[RunField] | None | Unset): Optional list of fields to include per run. Always
            includes: id, workflow_id, machine_id, status, created_at. Provide multiple 'fields='
            params to include more.
        skip (int | Unset):  Default: 0.
        limit (int | Unset):  Default: 100.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | PaginatedResponseRunResponse
    """

    return (
        await asyncio_detailed(
            client=client,
            workflow_id=workflow_id,
            machine_id=machine_id,
            session_id=session_id,
            status=status,
            created_at_from=created_at_from,
            created_at_to=created_at_to,
            fields=fields,
            skip=skip,
            limit=limit,
        )
    ).parsed



from http import HTTPStatus
from io import BytesIO
from typing import Any
from urllib.parse import quote
from uuid import UUID

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...types import File, Response


def _get_kwargs(
    attachment_id: UUID,
) -> dict[str, Any]:
    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/run-attachments/{attachment_id}/download".format(
            attachment_id=quote(str(attachment_id), safe=""),
        ),
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> File | HTTPValidationError | None:
    if response.status_code == 200:
        response_200 = File(payload=BytesIO(response.content))

        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[File | HTTPValidationError]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    attachment_id: UUID,
    *,
    client: AuthenticatedClient,
) -> Response[File | HTTPValidationError]:
    """Download Run Attachment

     Download a run attachment file.

    Returns the raw file content as a streaming response.

    Args:
        attachment_id (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[File | HTTPValidationError]
    """

    kwargs = _get_kwargs(
        attachment_id=attachment_id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    attachment_id: UUID,
    *,
    client: AuthenticatedClient,
) -> File | HTTPValidationError | None:
    """Download Run Attachment

     Download a run attachment file.

    Returns the raw file content as a streaming response.

    Args:
        attachment_id (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        File | HTTPValidationError
    """

    return sync_detailed(
        attachment_id=attachment_id,
        client=client,
    ).parsed


async def asyncio_detailed(
    attachment_id: UUID,
    *,
    client: AuthenticatedClient,
) -> Response[File | HTTPValidationError]:
    """Download Run Attachment

     Download a run attachment file.

    Returns the raw file content as a streaming response.

    Args:
        attachment_id (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[File | HTTPValidationError]
    """

    kwargs = _get_kwargs(
        attachment_id=attachment_id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    attachment_id: UUID,
    *,
    client: AuthenticatedClient,
) -> File | HTTPValidationError | None:
    """Download Run Attachment

     Download a run attachment file.

    Returns the raw file content as a streaming response.

    Args:
        attachment_id (UUID):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        File | HTTPValidationError
    """

    return (
        await asyncio_detailed(
            attachment_id=attachment_id,
            client=client,
        )
    ).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.workflow_model_metadata import WorkflowModelMetadata


T = TypeVar("T", bound="WorkflowUpdate")


@_attrs_define
class WorkflowUpdate:
    """Schema for updating a workflow

    Attributes:
        name (None | str | Unset):
        main_prompt (None | str | Unset):
        output_schema (None | str | Unset): JSON schema for output data transformation
        includes_file_exports (bool | None | Unset): Enable AI-based file export detection
        is_webhooks_enabled (bool | None | Unset): Send webhook on run completion
        model_metadata (None | Unset | WorkflowModelMetadata): Optional workflow-level model configuration metadata
            (main agent + fallbacks).
    """

    name: None | str | Unset = UNSET
    main_prompt: None | str | Unset = UNSET
    output_schema: None | str | Unset = UNSET
    includes_file_exports: bool | None | Unset = UNSET
    is_webhooks_enabled: bool | None | Unset = UNSET
    model_metadata: None | Unset | WorkflowModelMetadata = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.workflow_model_metadata import WorkflowModelMetadata

        name: None | str | Unset
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        main_prompt: None | str | Unset
        if isinstance(self.main_prompt, Unset):
            main_prompt = UNSET
        else:
            main_prompt = self.main_prompt

        output_schema: None | str | Unset
        if isinstance(self.output_schema, Unset):
            output_schema = UNSET
        else:
            output_schema = self.output_schema

        includes_file_exports: bool | None | Unset
        if isinstance(self.includes_file_exports, Unset):
            includes_file_exports = UNSET
        else:
            includes_file_exports = self.includes_file_exports

        is_webhooks_enabled: bool | None | Unset
        if isinstance(self.is_webhooks_enabled, Unset):
            is_webhooks_enabled = UNSET
        else:
            is_webhooks_enabled = self.is_webhooks_enabled

        model_metadata: dict[str, Any] | None | Unset
        if isinstance(self.model_metadata, Unset):
            model_metadata = UNSET
        elif isinstance(self.model_metadata, WorkflowModelMetadata):
            model_metadata = self.model_metadata.to_dict()
        else:
            model_metadata = self.model_metadata

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if main_prompt is not UNSET:
            field_dict["main_prompt"] = main_prompt
        if output_schema is not UNSET:
            field_dict["output_schema"] = output_schema
        if includes_file_exports is not UNSET:
            field_dict["includes_file_exports"] = includes_file_exports
        if is_webhooks_enabled is not UNSET:
            field_dict["is_webhooks_enabled"] = is_webhooks_enabled
        if model_metadata is not UNSET:
            field_dict["model_metadata"] = model_metadata

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.workflow_model_metadata import WorkflowModelMetadata

        d = dict(src_dict)

        def _parse_name(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        name = _parse_name(d.pop("name", UNSET))

        def _parse_main_prompt(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        main_prompt = _parse_main_prompt(d.pop("main_prompt", UNSET))

        def _parse_output_schema(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        output_schema = _parse_output_schema(d.pop("output_schema", UNSET))

        def _parse_includes_file_exports(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        includes_file_exports = _parse_includes_file_exports(d.pop("includes_file_exports", UNSET))

        def _parse_is_webhooks_enabled(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        is_webhooks_enabled = _parse_is_webhooks_enabled(d.pop("is_webhooks_enabled", UNSET))

        def _parse_model_metadata(data: object) -> None | Unset | WorkflowModelMetadata:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                model_metadata_type_0 = WorkflowModelMetadata.from_dict(data)

                return model_metadata_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | WorkflowModelMetadata, data)

        model_metadata = _parse_model_metadata(d.pop("model_metadata", UNSET))

        workflow_update = cls(
            name=name,
            main_prompt=main_prompt,
            output_schema=output_schema,
            includes_file_exports=includes_file_exports,
            is_webhooks_enabled=is_webhooks_enabled,
            model_metadata=model_metadata,
        )

        workflow_update.additional_properties = d
        return workflow_update

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.file_input import FileInput
    from ..models.run_bulk_create_input_values_type_0 import RunBulkCreateInputValuesType0
    from ..models.run_bulk_create_sensitive_input_values_type_0 import RunBulkCreateSensitiveInputValuesType0


T = TypeVar("T", bound="RunBulkCreate")


@_attrs_define
class RunBulkCreate:
    """Schema for bulk creating runs

    Attributes:
        workflow_id (UUID):
        count (int): Number of runs to create (max 1000)
        machine_id (None | Unset | UUID): Machine ID. If not provided, an available machine will be automatically
            selected.
        pool_ids (list[UUID] | None | Unset): Pool IDs to filter available machines. Machine must belong to all of these
            pools (intersection). Ignored when machine_id is provided.
        input_values (None | RunBulkCreateInputValuesType0 | Unset): Input values for workflow variables
        file_inputs (list[FileInput] | None | Unset): Files to upload to the machine
        sensitive_input_values (None | RunBulkCreateSensitiveInputValuesType0 | Unset): Sensitive input values (supports
            nested objects) to store in the secure vault per run. Not persisted in our database.
        session_id (None | Unset | UUID): Join an existing session; overrides machine_id/pool_ids for all runs
        start_session (bool | None | Unset): Start a new session for these runs; a new UUID will be generated and set on
            all runs. The first run will attempt to reserve a machine. Default: False.
    """

    workflow_id: UUID
    count: int
    machine_id: None | Unset | UUID = UNSET
    pool_ids: list[UUID] | None | Unset = UNSET
    input_values: None | RunBulkCreateInputValuesType0 | Unset = UNSET
    file_inputs: list[FileInput] | None | Unset = UNSET
    sensitive_input_values: None | RunBulkCreateSensitiveInputValuesType0 | Unset = UNSET
    session_id: None | Unset | UUID = UNSET
    start_session: bool | None | Unset = False
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.run_bulk_create_input_values_type_0 import RunBulkCreateInputValuesType0
        from ..models.run_bulk_create_sensitive_input_values_type_0 import RunBulkCreateSensitiveInputValuesType0

        workflow_id = str(self.workflow_id)

        count = self.count

        machine_id: None | str | Unset
        if isinstance(self.machine_id, Unset):
            machine_id = UNSET
        elif isinstance(self.machine_id, UUID):
            machine_id = str(self.machine_id)
        else:
            machine_id = self.machine_id

        pool_ids: list[str] | None | Unset
        if isinstance(self.pool_ids, Unset):
            pool_ids = UNSET
        elif isinstance(self.pool_ids, list):
            pool_ids = []
            for pool_ids_type_0_item_data in self.pool_ids:
                pool_ids_type_0_item = str(pool_ids_type_0_item_data)
                pool_ids.append(pool_ids_type_0_item)

        else:
            pool_ids = self.pool_ids

        input_values: dict[str, Any] | None | Unset
        if isinstance(self.input_values, Unset):
            input_values = UNSET
        elif isinstance(self.input_values, RunBulkCreateInputValuesType0):
            input_values = self.input_values.to_dict()
        else:
            input_values = self.input_values

        file_inputs: list[dict[str, Any]] | None | Unset
        if isinstance(self.file_inputs, Unset):
            file_inputs = UNSET
        elif isinstance(self.file_inputs, list):
            file_inputs = []
            for file_inputs_type_0_item_data in self.file_inputs:
                file_inputs_type_0_item = file_inputs_type_0_item_data.to_dict()
                file_inputs.append(file_inputs_type_0_item)

        else:
            file_inputs = self.file_inputs

        sensitive_input_values: dict[str, Any] | None | Unset
        if isinstance(self.sensitive_input_values, Unset):
            sensitive_input_values = UNSET
        elif isinstance(self.sensitive_input_values, RunBulkCreateSensitiveInputValuesType0):
            sensitive_input_values = self.sensitive_input_values.to_dict()
        else:
            sensitive_input_values = self.sensitive_input_values

        session_id: None | str | Unset
        if isinstance(self.session_id, Unset):
            session_id = UNSET
        elif isinstance(self.session_id, UUID):
            session_id = str(self.session_id)
        else:
            session_id = self.session_id

        start_session: bool | None | Unset
        if isinstance(self.start_session, Unset):
            start_session = UNSET
        else:
            start_session = self.start_session

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "workflow_id": workflow_id,
                "count": count,
            }
        )
        if machine_id is not UNSET:
            field_dict["machine_id"] = machine_id
        if pool_ids is not UNSET:
            field_dict["pool_ids"] = pool_ids
        if input_values is not UNSET:
            field_dict["input_values"] = input_values
        if file_inputs is not UNSET:
            field_dict["file_inputs"] = file_inputs
        if sensitive_input_values is not UNSET:
            field_dict["sensitive_input_values"] = sensitive_input_values
        if session_id is not UNSET:
            field_dict["session_id"] = session_id
        if start_session is not UNSET:
            field_dict["start_session"] = start_session

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.file_input import FileInput
        from ..models.run_bulk_create_input_values_type_0 import RunBulkCreateInputValuesType0
        from ..models.run_bulk_create_sensitive_input_values_type_0 import RunBulkCreateSensitiveInputValuesType0

        d = dict(src_dict)
        workflow_id = UUID(d.pop("workflow_id"))

        count = d.pop("count")

        def _parse_machine_id(data: object) -> None | Unset | UUID:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                machine_id_type_0 = UUID(data)

                return machine_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UUID, data)

        machine_id = _parse_machine_id(d.pop("machine_id", UNSET))

        def _parse_pool_ids(data: object) -> list[UUID] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                pool_ids_type_0 = []
                _pool_ids_type_0 = data
                for pool_ids_type_0_item_data in _pool_ids_type_0:
                    pool_ids_type_0_item = UUID(pool_ids_type_0_item_data)

                    pool_ids_type_0.append(pool_ids_type_0_item)

                return pool_ids_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[UUID] | None | Unset, data)

        pool_ids = _parse_pool_ids(d.pop("pool_ids", UNSET))

        def _parse_input_values(data: object) -> None | RunBulkCreateInputValuesType0 | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                input_values_type_0 = RunBulkCreateInputValuesType0.from_dict(data)

                return input_values_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | RunBulkCreateInputValuesType0 | Unset, data)

        input_values = _parse_input_values(d.pop("input_values", UNSET))

        def _parse_file_inputs(data: object) -> list[FileInput] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                file_inputs_type_0 = []
                _file_inputs_type_0 = data
                for file_inputs_type_0_item_data in _file_inputs_type_0:
                    file_inputs_type_0_item = FileInput.from_dict(file_inputs_type_0_item_data)

                    file_inputs_type_0.append(file_inputs_type_0_item)

                return file_inputs_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[FileInput] | None | Unset, data)

        file_inputs = _parse_file_inputs(d.pop("file_inputs", UNSET))

        def _parse_sensitive_input_values(data: object) -> None | RunBulkCreateSensitiveInputValuesType0 | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                sensitive_input_values_type_0 = RunBulkCreateSensitiveInputValuesType0.from_dict(data)

                return sensitive_input_values_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | RunBulkCreateSensitiveInputValuesType0 | Unset, data)

        sensitive_input_values = _parse_sensitive_input_values(d.pop("sensitive_input_values", UNSET))

        def _parse_session_id(data: object) -> None | Unset | UUID:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                session_id_type_0 = UUID(data)

                return session_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UUID, data)

        session_id = _parse_session_id(d.pop("session_id", UNSET))

        def _parse_start_session(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        start_session = _parse_start_session(d.pop("start_session", UNSET))

        run_bulk_create = cls(
            workflow_id=workflow_id,
            count=count,
            machine_id=machine_id,
            pool_ids=pool_ids,
            input_values=input_values,
            file_inputs=file_inputs,
            sensitive_input_values=sensitive_input_values,
            session_id=session_id,
            start_session=start_session,
        )

        run_bulk_create.additional_properties = d
        return run_bulk_create

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="WorkflowModelMetadata")


@_attrs_define
class WorkflowModelMetadata:
    """JSONB-backed workflow model configuration metadata.

    Stored on the Workflow row as `model_metadata` to avoid adding many FK columns.
    All fields are optional; when missing/null, the system falls back to Cyberdesk defaults.

        Attributes:
            main_agent_model_id (None | Unset | UUID): ModelConfiguration.id used for the main agent. Null → Cyberdesk
                default.
            fallback_model_1_id (None | Unset | UUID): ModelConfiguration.id used as fallback 1 (global across agents).
            fallback_model_2_id (None | Unset | UUID): ModelConfiguration.id used as fallback 2 (global across agents).
    """

    main_agent_model_id: None | Unset | UUID = UNSET
    fallback_model_1_id: None | Unset | UUID = UNSET
    fallback_model_2_id: None | Unset | UUID = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        main_agent_model_id: None | str | Unset
        if isinstance(self.main_agent_model_id, Unset):
            main_agent_model_id = UNSET
        elif isinstance(self.main_agent_model_id, UUID):
            main_agent_model_id = str(self.main_agent_model_id)
        else:
            main_agent_model_id = self.main_agent_model_id

        fallback_model_1_id: None | str | Unset
        if isinstance(self.fallback_model_1_id, Unset):
            fallback_model_1_id = UNSET
        elif isinstance(self.fallback_model_1_id, UUID):
            fallback_model_1_id = str(self.fallback_model_1_id)
        else:
            fallback_model_1_id = self.fallback_model_1_id

        fallback_model_2_id: None | str | Unset
        if isinstance(self.fallback_model_2_id, Unset):
            fallback_model_2_id = UNSET
        elif isinstance(self.fallback_model_2_id, UUID):
            fallback_model_2_id = str(self.fallback_model_2_id)
        else:
            fallback_model_2_id = self.fallback_model_2_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if main_agent_model_id is not UNSET:
            field_dict["main_agent_model_id"] = main_agent_model_id
        if fallback_model_1_id is not UNSET:
            field_dict["fallback_model_1_id"] = fallback_model_1_id
        if fallback_model_2_id is not UNSET:
            field_dict["fallback_model_2_id"] = fallback_model_2_id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_main_agent_model_id(data: object) -> None | Unset | UUID:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                main_agent_model_id_type_0 = UUID(data)

                return main_agent_model_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UUID, data)

        main_agent_model_id = _parse_main_agent_model_id(d.pop("main_agent_model_id", UNSET))

        def _parse_fallback_model_1_id(data: object) -> None | Unset | UUID:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                fallback_model_1_id_type_0 = UUID(data)

                return fallback_model_1_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UUID, data)

        fallback_model_1_id = _parse_fallback_model_1_id(d.pop("fallback_model_1_id", UNSET))

        def _parse_fallback_model_2_id(data: object) -> None | Unset | UUID:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                fallback_model_2_id_type_0 = UUID(data)

                return fallback_model_2_id_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UUID, data)

        fallback_model_2_id = _parse_fallback_model_2_id(d.pop("fallback_model_2_id", UNSET))

        workflow_model_metadata = cls(
            main_agent_model_id=main_agent_model_id,
            fallback_model_1_id=fallback_model_1_id,
            fallback_model_2_id=fallback_model_2_id,
        )

        workflow_model_metadata.additional_properties = d
        return workflow_model_metadata

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="WorkflowChainResponse")


@_attrs_define
class WorkflowChainResponse:
    """Response for chain creation

    Attributes:
        session_id (UUID):
        run_ids (list[UUID]):
    """

    session_id: UUID
    run_ids: list[UUID]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        session_id = str(self.session_id)

        run_ids = []
        for run_ids_item_data in self.run_ids:
            run_ids_item = str(run_ids_item_data)
            run_ids.append(run_ids_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "session_id": session_id,
                "run_ids": run_ids,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        session_id = UUID(d.pop("session_id"))

        run_ids = []
        _run_ids = d.pop("run_ids")
        for run_ids_item_data in _run_ids:
            run_ids_item = UUID(run_ids_item_data)

            run_ids.append(run_ids_item)

        workflow_chain_response = cls(
            session_id=session_id,
            run_ids=run_ids,
        )

        workflow_chain_response.additional_properties = d
        return workflow_chain_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.chain_step_inputs_type_0 import ChainStepInputsType0
    from ..models.chain_step_sensitive_inputs_type_0 import ChainStepSensitiveInputsType0


T = TypeVar("T", bound="ChainStep")


@_attrs_define
class ChainStep:
    """One step within a chain

    Attributes:
        workflow_id (UUID):
        session_alias (None | str | Unset): Alias to persist this step's outputs within the session
        inputs (ChainStepInputsType0 | None | Unset): Step-specific inputs; values can be strings, objects, arrays, or
            {$ref: 'alias.outputs.path'} references
        sensitive_inputs (ChainStepSensitiveInputsType0 | None | Unset): Step-specific sensitive inputs (supports nested
            objects) that override or extend shared_sensitive_inputs
    """

    workflow_id: UUID
    session_alias: None | str | Unset = UNSET
    inputs: ChainStepInputsType0 | None | Unset = UNSET
    sensitive_inputs: ChainStepSensitiveInputsType0 | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.chain_step_inputs_type_0 import ChainStepInputsType0
        from ..models.chain_step_sensitive_inputs_type_0 import ChainStepSensitiveInputsType0

        workflow_id = str(self.workflow_id)

        session_alias: None | str | Unset
        if isinstance(self.session_alias, Unset):
            session_alias = UNSET
        else:
            session_alias = self.session_alias

        inputs: dict[str, Any] | None | Unset
        if isinstance(self.inputs, Unset):
            inputs = UNSET
        elif isinstance(self.inputs, ChainStepInputsType0):
            inputs = self.inputs.to_dict()
        else:
            inputs = self.inputs

        sensitive_inputs: dict[str, Any] | None | Unset
        if isinstance(self.sensitive_inputs, Unset):
            sensitive_inputs = UNSET
        elif isinstance(self.sensitive_inputs, ChainStepSensitiveInputsType0):
            sensitive_inputs = self.sensitive_inputs.to_dict()
        else:
            sensitive_inputs = self.sensitive_inputs

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "workflow_id": workflow_id,
            }
        )
        if session_alias is not UNSET:
            field_dict["session_alias"] = session_alias
        if inputs is not UNSET:
            field_dict["inputs"] = inputs
        if sensitive_inputs is not UNSET:
            field_dict["sensitive_inputs"] = sensitive_inputs

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.chain_step_inputs_type_0 import ChainStepInputsType0
        from ..models.chain_step_sensitive_inputs_type_0 import ChainStepSensitiveInputsType0

        d = dict(src_dict)
        workflow_id = UUID(d.pop("workflow_id"))

        def _parse_session_alias(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        session_alias = _parse_session_alias(d.pop("session_alias", UNSET))

        def _parse_inputs(data: object) -> ChainStepInputsType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                inputs_type_0 = ChainStepInputsType0.from_dict(data)

                return inputs_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ChainStepInputsType0 | None | Unset, data)

        inputs = _parse_inputs(d.pop("inputs", UNSET))

        def _parse_sensitive_inputs(data: object) -> ChainStepSensitiveInputsType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                sensitive_inputs_type_0 = ChainStepSensitiveInputsType0.from_dict(data)

                return sensitive_inputs_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ChainStepSensitiveInputsType0 | None | Unset, data)

        sensitive_inputs = _parse_sensitive_inputs(d.pop("sensitive_inputs", UNSET))

        chain_step = cls(
            workflow_id=workflow_id,
            session_alias=session_alias,
            inputs=inputs,
            sensitive_inputs=sensitive_inputs,
        )

        chain_step.additional_properties = d
        return chain_step

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="MachinePoolUpdate")


@_attrs_define
class MachinePoolUpdate:
    """Schema for updating a machine's pool assignments

    Attributes:
        pool_ids (list[UUID]): List of pool IDs to assign the machine to
    """

    pool_ids: list[UUID]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        pool_ids = []
        for pool_ids_item_data in self.pool_ids:
            pool_ids_item = str(pool_ids_item_data)
            pool_ids.append(pool_ids_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "pool_ids": pool_ids,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        pool_ids = []
        _pool_ids = d.pop("pool_ids")
        for pool_ids_item_data in _pool_ids:
            pool_ids_item = UUID(pool_ids_item_data)

            pool_ids.append(pool_ids_item)

        machine_pool_update = cls(
            pool_ids=pool_ids,
        )

        machine_pool_update.additional_properties = d
        return machine_pool_update

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ConnectionCreate")


@_attrs_define
class ConnectionCreate:
    """Schema for creating a connection

    Attributes:
        websocket_id (str):
        machine_id (UUID):
        ip_address (None | str | Unset):
        user_agent (None | str | Unset):
    """

    websocket_id: str
    machine_id: UUID
    ip_address: None | str | Unset = UNSET
    user_agent: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        websocket_id = self.websocket_id

        machine_id = str(self.machine_id)

        ip_address: None | str | Unset
        if isinstance(self.ip_address, Unset):
            ip_address = UNSET
        else:
            ip_address = self.ip_address

        user_agent: None | str | Unset
        if isinstance(self.user_agent, Unset):
            user_agent = UNSET
        else:
            user_agent = self.user_agent

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "websocket_id": websocket_id,
                "machine_id": machine_id,
            }
        )
        if ip_address is not UNSET:
            field_dict["ip_address"] = ip_address
        if user_agent is not UNSET:
            field_dict["user_agent"] = user_agent

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        websocket_id = d.pop("websocket_id")

        machine_id = UUID(d.pop("machine_id"))

        def _parse_ip_address(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        ip_address = _parse_ip_address(d.pop("ip_address", UNSET))

        def _parse_user_agent(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        user_agent = _parse_user_agent(d.pop("user_agent", UNSET))

        connection_create = cls(
            websocket_id=websocket_id,
            machine_id=machine_id,
            ip_address=ip_address,
            user_agent=user_agent,
        )

        connection_create.additional_properties = d
        return connection_create

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
